# -*- coding: utf-8 -*-

# Copyright 2013 Sebastian Ramacher <sebastian+dev@ramacher.at>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

""" python-discid compat layer

This module provides a compatible layer so that python-libdiscid can be used as
a replacement for python-discid. It provides an interface compatible with
python-discid version 1.0.2.
"""

import libdiscid
import operator
import functools
import sys

try:
  unicode
except NameError:
  # 2/3 compat
  unicode = str

_INVERSE_FEATURES= {
    libdiscid.FEATURES_MAPPING[libdiscid.FEATURE_READ]: libdiscid.FEATURE_READ,
    libdiscid.FEATURES_MAPPING[libdiscid.FEATURE_MCN]: libdiscid.FEATURE_MCN,
    libdiscid.FEATURES_MAPPING[libdiscid.FEATURE_ISRC]: libdiscid.FEATURE_ISRC
  }

class _NoneHelper(object):
  def __getattr__(self, name):
    if name in ('id', 'freedb_id', 'submission_url', 'toc',
                'first_track', 'last_track', 'sectors', 'mcn'):
      return None

    return super(_NoneHelper, self).__getattr__(name)

def _decode(string, encoding=None):
  # Let's do the same thing discid is doing. It always accepts both strings and
  # unicode objects and encodes/decodes them as it sees fit. libdiscid always
  # wants unicode objects, so let's decode it here on a best effort basis.
  if not isinstance(string, unicode):
    if encoding is None:
      encoding = sys.getfilesystemencoding() or 'ascii'
    return string.decode(encoding)
  return string

# exceptions defined in discid
DiscError = libdiscid.DiscError

class TOCError(Exception):
  pass

# classes defined in discid
class Track(object):
  def __init__(self, disc, number):
    self.disc = disc
    self.number = number

  def __str__(self):
    return str(self.number)

  @property
  def offset(self):
    return self.disc.track_offsets[self.number - self.disc.first_track]

  @property
  def sectors(self):
    return self.disc.track_lengths[self.number - self.disc.first_track]
  length = sectors

  @property
  def seconds(self):
    return libdiscid.sectors_to_seconds(self.sectors)

  @property
  def isrc(self):
    try:
      value = self.disc.track_isrcs[self.number - self.disc.first_track]
    except NotImplementedError:
      return None
    return value if value != '' else None

class Disc(object):
  def __init__(self):
    self.disc = _NoneHelper()
    self.tracks = []

  def read(self, device, features=[]):
    self.disc = libdiscid.read(device, functools.reduce(
      operator.or_, (
        _INVERSE_FEATURES[feature]
        for feature in features if feature in FEATURES
      ), 0))
    self.tracks = [
      Track(self.disc, numb) for numb in range(self.disc.first_track,
                                               self.disc.last_track + 1)]
    return True

  def put(self, first, last, disc_sectors, track_offsets):
    try:
      self.disc = libdiscid.put(first, last, disc_sectors, track_offsets)
    except DiscError as disc_error:
      raise TOCError(str(disc_error))

    self.tracks = [
      Track(self.disc, num) for num in range(self.disc.first_track,
                                             self.disc.last_track + 1)]
    return True

  @property
  def id(self):
    return self.disc.id

  @property
  def freedb_id(self):
    return self.disc.freedb_id

  @property
  def submission_url(self):
    return self.disc.submission_url

  @property
  def toc_string(self):
    return self.disc.toc

  @property
  def first_track_num(self):
    return self.disc.first_track

  @property
  def last_track_num(self):
    return self.disc.last_track

  @property
  def sectors(self):
    return self.disc.sectors
  length = sectors

  @property
  def seconds(self):
    return libdiscid.sectors_to_seconds(self.sectors) \
        if self.sectors is not None else None

  @property
  def mcn(self):
    try:
      value = self.disc.mcn
    except NotImplementedError:
      return None
    return value if value != '' else None

# functions defined in discid
get_default_device = libdiscid.default_device

def read(device=None, features=[]):
  disc = Disc()
  disc.read(_decode(device) if device is not None else None,
            map(lambda feature: _decode(feature, 'ascii'), features))
  return disc

def put(first, last, disc_sectors, track_offsets):
  disc = Disc()
  disc.put(first, last, disc_sectors, track_offsets)
  return disc

# constants defined in discid
__version__ = '1.1.0 (compat layer from python-libdiscid %s)' % \
  (libdiscid.__version__, )
""" This is the version of python-discid this layer is compatible with. """

LIBDISCID_VERSION_STRING = libdiscid.__discid_version__
FEATURES = libdiscid.FEATURES
FEATURES_IMPLEMENTED = (libdiscid.FEATURE_READ, libdiscid.FEATURE_MCN,
                        libdiscid.FEATURE_ISRC)

__all__ = (
  'read', 'put', 'get_default_device',
  '__version__', 'LIBDISCID_VERSION_STRING', 'FEATURES', 'FEATURES_IMPLEMENTED',
  'Disc', 'DiscError', 'TOCError'
)



# -*- coding: utf-8 -*-

# Copyright 2013 Sebastian Ramacher <sebastian+dev@ramacher.at>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

""" Tests for the libdiscid.compat.discid module
"""

from __future__ import unicode_literals

import unittest
import libdiscid
from libdiscid.compat import discid
from libdiscid.compat.discid import DiscError, TOCError


class TestCompatDiscID(unittest.TestCase):
  def test_default_device(self):
    self.assertIsNotNone(discid.get_default_device())

  def test_features(self):
    self.assertIsNotNone(discid.FEATURES)

  def test_features_implementes(self):
    self.assertIsNotNone(discid.FEATURES_IMPLEMENTED)

  def test_empty_is_none(self):
    disc = discid.Disc()
    self.assertIsNone(disc.id)
    self.assertIsNone(disc.freedb_id)
    self.assertIsNone(disc.submission_url)
    self.assertIsNone(disc.toc_string)
    self.assertIsNone(disc.first_track_num)
    self.assertIsNone(disc.last_track_num)
    self.assertIsNone(disc.sectors)
    self.assertIsNone(disc.seconds)
    self.assertEqual(len(disc.tracks), 0)

  @unittest.skipIf(libdiscid.FEATURES_MAPPING[libdiscid.FEATURE_READ] not in
                   libdiscid.FEATURES, 'not available on this platform')
  def test_read_fail(self):
    self.assertRaises(DiscError, discid.read, '/does/not/exist')

  def test_read_None(self):
    try:
      discid.read()
    except (DiscError, NotImplementedError):
      pass

  @unittest.skipIf(libdiscid.FEATURES_MAPPING[libdiscid.FEATURE_READ] not in
                   libdiscid.FEATURES, 'not available on this platform')
  def test_encoded_device(self):
    self.assertRaises(DiscError, discid.read, '/does/not/exist')

  @unittest.skipIf(libdiscid.FEATURES_MAPPING[libdiscid.FEATURE_READ] not in
                   libdiscid.FEATURES, 'not available on this platform')
  def test_byte_device(self):
    self.assertRaises(DiscError, discid.read, b'/does/not/exist')

  def test_put(self):
    testdata = libdiscid.tests.common.PutSuccess

    disc = discid.put(testdata.first, testdata.last, testdata.sectors,
                      testdata.offsets)
    self.assertIsNotNone(disc)

    self.assertEqual(disc.id, testdata.disc_id)
    self.assertEqual(disc.freedb_id, testdata.freedb_id)
    self.assertIsNotNone(disc.submission_url)
    self.assertEqual(disc.toc_string, testdata.toc)
    self.assertEqual(disc.first_track_num, testdata.first)
    self.assertEqual(disc.last_track_num, testdata.last)
    self.assertEqual(disc.sectors, testdata.sectors)
    self.assertEqual(disc.seconds, testdata.seconds)

    self.assertEqual(len(disc.tracks), len(testdata.offsets))
    for track, offset, sec in zip(disc.tracks, testdata.offsets,
                                  testdata.track_seconds):
      self.assertEqual(track.offset, offset)
      self.assertEqual(track.seconds, sec)

    # ISRCs are not available if one calls put
    for track in disc.tracks:
        self.assertIsNone(track.isrc)

    # MCN is not available if one calls put
    self.assertIsNone(disc.mcn)

  def test_put_fail_1(self):
    # !(first < last)
    testdata = libdiscid.tests.common.PutFail1
    self.assertRaises(TOCError, discid.put, testdata.first, testdata.last,
                      testdata.sectors, testdata.offsets)

  def test_put_fail_2(self):
    # !(first >= 1)
    testdata = libdiscid.tests.common.PutFail2
    self.assertRaises(TOCError, discid.put, testdata.first, testdata.last,
                      testdata.sectors, testdata.offsets)

    # !(first < 100)
    testdata = libdiscid.tests.common.PutFail2_2
    self.assertRaises(TOCError, discid.put, testdata.first, testdata.last,
                      testdata.sectors, testdata.offsets)

  def test_put_fail_3(self):
    # !(last >= 1)
    testdata = libdiscid.tests.common.PutFail3
    self.assertRaises(TOCError, discid.put, testdata.first, testdata.last,
                      testdata.sectors, testdata.offsets)

    # !(last < 100)
    testdata = libdiscid.tests.common.PutFail3_2
    self.assertRaises(TOCError, discid.put, testdata.first, testdata.last,
                      testdata.sectors, testdata.offsets)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python

import os.path
import sys

from setuptools import setup, Extension

try:
  from Cython.Build import cythonize
  have_cython = True
except ImportError:
  have_cython = False

try:
  import pkgconfig
  have_pkgconfig = True
except ImportError:
  have_pkgconfig = False

if have_pkgconfig and pkgconfig.exists('libdiscid'):
  flags = pkgconfig.parse('libdiscid')
  define_macros = flags['define_macros']
  include_dirs = flags['include_dirs']
  library_dirs = flags['library_dirs']
  libraries = list(flags['libraries'])
else:
  define_macros = ''
  include_dirs = ''
  library_dirs = ''
  libraries = 'discid'

if have_cython:
  # if Cython is available, rebuild _discid.c
  ext = cythonize([
    Extension('libdiscid._discid',
      [
        'libdiscid/_discid.pyx',
        'libdiscid/discid-wrapper.c'
      ],
      define_macros=define_macros,
      include_dirs=include_dirs,
      library_dirs=library_dirs,
      libraries=libraries
    )
  ])
else:
  # ... otherwise use the shipped version of _discid.c
  ext = [
    Extension('libdiscid._discid',
      [
        'libdiscid/_discid.c',
        'libdiscid/discid-wrapper.c'
      ],
      define_macros=define_macros,
      include_dirs=include_dirs,
      library_dirs=library_dirs,
      libraries=libraries
    )
  ]

setup_requires = ['pkgconfig']
if have_cython:
  # if Cython is available, check if it's new enough
  setup_requires.append('cython >= 0.15')

def read(name):
  f = open(os.path.join(os.path.dirname(__file__), name))
  ret = f.read()
  f.close()
  return ret

setup(
  name='python-libdiscid',
  version='1.0',
  description='Python bindings for libdiscid',
  long_description=read('README.rst'),
  author='Sebastian Ramacher',
  author_email='sebastian+dev@ramacher.at',
  url='https://github.com/sebastinas/python-libdiscid',
  license='Expat',
  ext_modules=ext,
  packages=[
    'libdiscid',
    'libdiscid.tests',
    'libdiscid.compat'
  ],
  classifiers=[
    'Development Status :: 5 - Production/Stable',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Topic :: Multimedia :: Sound/Audio :: CD Audio :: CD Ripping',
    'Topic :: Software Development :: Libraries :: Python Modules'
  ],
  test_suite='libdiscid.tests',
  setup_requires=setup_requires
)
